@@ -1,6 +1,7 @@
 """This file implements the gym environment of Rex.
 
 """
+import collections
 import math
 import time
 import gym
@@ -11,14 +12,11 @@ import pybullet_data
 from gym import spaces
 from gym.utils import seeding
 
-from ..model import rex, motor
+from ..model import rex, motor, mark_constants, rex_constants
+from ..model.terrain import Terrain
 from ..util import bullet_client
 
-NUM_MOTORS = 12
 MOTOR_ANGLE_OBSERVATION_INDEX = 0
-MOTOR_VELOCITY_OBSERVATION_INDEX = MOTOR_ANGLE_OBSERVATION_INDEX + NUM_MOTORS
-MOTOR_TORQUE_OBSERVATION_INDEX = MOTOR_VELOCITY_OBSERVATION_INDEX + NUM_MOTORS
-BASE_ORIENTATION_OBSERVATION_INDEX = MOTOR_TORQUE_OBSERVATION_INDEX + NUM_MOTORS
 OBSERVATION_EPS = 0.01
 RENDER_HEIGHT = 360
 RENDER_WIDTH = 480
@@ -52,11 +50,12 @@ class RexGymEnv(gym.Env):
     metadata = {"render.modes": ["human", "rgb_array"], "video.frames_per_second": 100}
 
     def __init__(self,
+                 debug=False,
                  urdf_root=pybullet_data.getDataPath(),
                  urdf_version=None,
                  distance_weight=1.0,
-                 energy_weight=0.005,
-                 shake_weight=0.0,
+                 energy_weight=0.0005,
+                 shake_weight=0.005,
                  drift_weight=2.0,
                  distance_limit=float("inf"),
                  observation_noise_stdev=SENSOR_NOISE_STDDEV,
@@ -66,8 +65,8 @@ class RexGymEnv(gym.Env):
                  leg_model_enabled=True,
                  accurate_motor_model_enabled=False,
                  remove_default_joint_damping=False,
-                 motor_kp=1.0,
-                 motor_kd=0.02,
+                 motor_kp=2.0,
+                 motor_kd=0.03,
                  control_latency=0.0,
                  pd_latency=0.0,
                  torque_control_enabled=False,
@@ -83,7 +82,23 @@ class RexGymEnv(gym.Env):
                  reflection=True,
                  log_path=None,
                  target_orient=None,
-                 init_orient=None):
+                 init_orient=None,
+                 target_position=None,
+                 start_position=None,
+                 base_y=0.0,
+                 base_z=0.0,
+                 base_roll=0.0,
+                 base_pitch=0.0,
+                 base_yaw=0.0,
+                 step_length=None,
+                 step_rotation=None,
+                 step_angle=None,
+                 step_period=None,
+                 backwards=None,
+                 signal_type="ik",
+                 terrain_type="plane",
+                 terrain_id=None,
+                 mark='base'):
         """ Initialize the rex gym environment.
 
             Args:
@@ -123,8 +138,8 @@ class RexGymEnv(gym.Env):
                 is called. If set to false, reset just place Rex back to start
                 position and set its pose to initial configuration.
               on_rack: Whether to place Rex on rack. This is only used to debug
-                the walking gait. In this mode, Rex's base is hanged midair so
-                that its walking gait is clearer to visualize.
+                the walk gait. In this mode, Rex's base is hanged midair so
+                that its walk gait is clearer to visualize.
               render: Whether to render the simulation.
               num_steps_to_log: The max number of control steps in one episode that will
                 be logged. If the number of steps is more than num_steps_to_log, the
@@ -142,6 +157,11 @@ class RexGymEnv(gym.Env):
             Raises:
               ValueError: If the urdf_version is not supported.
         """
+        self.mark = mark
+        self.num_motors = mark_constants.MARK_DETAILS['motors_num'][self.mark]
+        self.motor_velocity_obs_index = MOTOR_ANGLE_OBSERVATION_INDEX + self.num_motors
+        self.motor_torque_obs_index = self.motor_velocity_obs_index + self.num_motors
+        self.base_orientation_obs_index = self.motor_torque_obs_index + self.num_motors
         # Set up logging.
         self._log_path = log_path
         # @TODO fix logging
@@ -172,6 +192,7 @@ class RexGymEnv(gym.Env):
         self._env_step_counter = 0
         self._num_steps_to_log = num_steps_to_log
         self._is_render = render
+        self._is_debug = debug
         self._last_base_position = [0, 0, 0]
         self._last_base_orientation = [0, 0, 0, 1]
         self._distance_weight = distance_weight
@@ -211,21 +232,57 @@ class RexGymEnv(gym.Env):
         if self._urdf_version is None:
             self._urdf_version = DEFAULT_URDF_VERSION
         self._pybullet_client.setPhysicsEngineParameter(enableConeFriction=0)
+        self._signal_type = signal_type
+        # gait inputs
+        self.step_length = step_length
+        self.step_rotation = step_rotation
+        self.step_angle = step_angle
+        self.step_period = step_period
+        # poses inputs
+        self._base_x = 0.01
+        self._base_y = base_y
+        self._base_z = base_z
+        self._base_roll = base_roll
+        self._base_pitch = base_pitch
+        self._base_yaw = base_yaw
+        # envs inputs
         self._target_orient = target_orient
         self._init_orient = init_orient
-        self._random_target = False
-        self._random_start = False
+        self._target_position = target_position
+        self._start_position = start_position
+        # computation support params
+        self._random_pos_target = False
+        self._random_pos_start = False
+        self._random_orient_target = False
+        self._random_orient_start = False
+        self._companion_obj = {}
+        self._queue = collections.deque(["base_y", "base_z", "roll", "pitch", "yaw"])
+        self._ranges = {
+            "base_x": (-0.02, 0.02, 0.01),
+            "base_y": (-0.007, 0.007, 0),
+            "base_z": (-0.048, 0.021, 0),
+            "roll": (-np.pi / 4, np.pi / 4, 0),
+            "pitch": (-np.pi / 4, np.pi / 4, 0),
+            "yaw": (-np.pi / 4, np.pi / 4, 0)
+        }
         self.seed()
+        self._backwards = backwards
+        self._terrain_type = "plane"
+        self._terrain_id = terrain_id
         self.reset()
+        self._terrain_type = terrain_type
+        self.terrain = Terrain(self._terrain_type, self._terrain_id)
+        if self._terrain_type is not "plane":
+            self.terrain.generate_terrain(self)
         observation_high = (self._get_observation_upper_bound() + OBSERVATION_EPS)
         observation_low = (self._get_observation_lower_bound() - OBSERVATION_EPS)
-        action_dim = NUM_MOTORS
+        action_dim = self.num_motors
         action_high = np.array([self._action_bound] * action_dim)
         self.action_space = spaces.Box(-action_high, action_high)
         self.observation_space = spaces.Box(observation_low, observation_high)
         self.viewer = None
         self._hard_reset = hard_reset  # This assignment need to be after reset()
-        self.goal_reached = False
+        self.env_goal_reached = False
 
     def close(self):
         # @TODO fix logger
@@ -237,6 +294,7 @@ class RexGymEnv(gym.Env):
         self._env_randomizers.append(env_randomizer)
 
     def reset(self, initial_motor_angles=None, reset_duration=1.0):
+        self.env_goal_reached = False
         self._pybullet_client.configureDebugVisualizer(self._pybullet_client.COV_ENABLE_RENDERING, 0)
         # @TODO fix logger
         # if self._env_step_counter > 0:
@@ -276,7 +334,9 @@ class RexGymEnv(gym.Env):
                     observation_noise_stdev=self._observation_noise_stdev,
                     torque_control_enabled=self._torque_control_enabled,
                     motor_overheat_protection=motor_protect,
-                    on_rack=self._on_rack))
+                    on_rack=self._on_rack,
+                    terrain_id=self._terrain_id,
+                    mark=self.mark))
         self.rex.Reset(reload_urdf=False,
                        default_motor_angles=initial_motor_angles,
                        reset_time=reset_duration)
@@ -284,7 +344,8 @@ class RexGymEnv(gym.Env):
         # Loop over all env randomizers.
         for env_randomizer in self._env_randomizers:
             env_randomizer.randomize_env(self)
-
+        if self._terrain_type is not "plane":
+            self.terrain.update_terrain()
         self._pybullet_client.setPhysicsEngineParameter(enableConeFriction=0)
         self._env_step_counter = 0
         self._last_base_position = [0, 0, 0]
@@ -300,7 +361,10 @@ class RexGymEnv(gym.Env):
         return [seed]
 
     def _transform_action_to_motor_command(self, action):
-        pass
+        if len(action) != mark_constants.MARK_DETAILS['motors_num'][self.mark]:
+            # extend with arm rest pose
+            action = np.concatenate((action, rex_constants.ARM_POSES["rest"]))
+        return action
 
     def step(self, action):
         """Step forward the simulation, given the action.
@@ -380,7 +444,7 @@ class RexGymEnv(gym.Env):
         Returns:
           A numpy array of motor angles.
         """
-        return np.array(self._observation[MOTOR_ANGLE_OBSERVATION_INDEX:MOTOR_ANGLE_OBSERVATION_INDEX + NUM_MOTORS])
+        return np.array(self._observation[MOTOR_ANGLE_OBSERVATION_INDEX:MOTOR_ANGLE_OBSERVATION_INDEX + self.num_motors])
 
     def get_rex_motor_velocities(self):
         """Get the rex's motor velocities.
@@ -389,7 +453,7 @@ class RexGymEnv(gym.Env):
           A numpy array of motor velocities.
         """
         return np.array(
-            self._observation[MOTOR_VELOCITY_OBSERVATION_INDEX:MOTOR_VELOCITY_OBSERVATION_INDEX + NUM_MOTORS])
+            self._observation[self.motor_velocity_obs_index:self.motor_velocity_obs_index + self.num_motors])
 
     def get_rex_motor_torques(self):
         """Get the rex's motor torques.
@@ -398,7 +462,7 @@ class RexGymEnv(gym.Env):
           A numpy array of motor torques.
         """
         return np.array(
-            self._observation[MOTOR_TORQUE_OBSERVATION_INDEX:MOTOR_TORQUE_OBSERVATION_INDEX + NUM_MOTORS])
+            self._observation[self.motor_torque_obs_index:self.motor_torque_obs_index + self.num_motors])
 
     def get_rex_base_orientation(self):
         """Get the rex's base orientation, represented by a quaternion.
@@ -406,7 +470,7 @@ class RexGymEnv(gym.Env):
         Returns:
           A numpy array of rex's orientation.
         """
-        return np.array(self._observation[BASE_ORIENTATION_OBSERVATION_INDEX:])
+        return np.array(self._observation[self.base_orientation_obs_index:])
 
     def is_fallen(self):
         """Decide whether Rex has fallen.
@@ -425,25 +489,49 @@ class RexGymEnv(gym.Env):
 
     def _termination(self):
         if self.is_fallen():
-            print("IS FALLING DOWN!")
-        o = self.rex.GetBaseOrientation()
-        if o[1] < -0.13:
-            print("IS ROTATING!")
-        return self.is_fallen() or o[1] < -0.13
+            print("FALLING DOWN!")
+        if self._out_of_trajectory():
+            print("OUT OF TRAJECTORY!")
+        return self.is_fallen() or self.env_goal_reached or self._out_of_trajectory()
+
+    @staticmethod
+    def _out_of_trajectory():
+        return False
 
     def _reward(self):
         current_base_position = self.rex.GetBasePosition()
-        # forward direction
-        forward_reward = -current_base_position[0] + self._last_base_position[0]
+        # observation = self._get_observation()
+        # forward gait
+        current_x = -current_base_position[0]
+        if self._backwards:
+            # backwards gait
+            current_x = -current_x
+        if self._target_position is not None:
+            self._target_position = abs(self._target_position)
+            # 0.15 tolerance
+            if current_x > self._target_position + 0.15:
+                forward_reward = self._target_position - current_x
+            elif self._target_position <= current_x <= self._target_position + 0.15:
+                forward_reward = 1.0
+            # stationary reward must be null: tolerance 5%
+            elif current_x <= 0.05:
+                forward_reward = 0.0
+            else:
+                forward_reward = current_x / self._target_position
+        else:
+            # the far the better..
+            forward_reward = current_x
         # Cap the forward reward if a cap is set.
         forward_reward = min(forward_reward, self._forward_reward_cap)
         # Penalty for sideways translation.
-        drift_reward = -abs(current_base_position[1] - self._last_base_position[1])
+        # drift_reward = -abs(current_base_position[1] - self._last_base_position[1])
+        drift_reward = -abs(current_base_position[1])
         # Penalty for sideways rotation of the body.
         orientation = self.rex.GetBaseOrientation()
         rot_matrix = pybullet.getMatrixFromQuaternion(orientation)
         local_up_vec = rot_matrix[6:]
         shake_reward = -abs(np.dot(np.asarray([1, 1, 0]), np.asarray(local_up_vec)))
+        # shake_reward = -abs(observation[4])
         energy_reward = -np.abs(
             np.dot(self.rex.GetMotorTorques(),
                    self.rex.GetMotorVelocities())) * self._time_step
@@ -515,8 +603,8 @@ class RexGymEnv(gym.Env):
         upper_bound = np.zeros(self._get_observation_dimension())
         num_motors = self.rex.num_motors
         upper_bound[0:num_motors] = math.pi  # Joint angle.
-        upper_bound[num_motors:2 * num_motors] = (motor.MOTOR_SPEED_LIMIT)  # Joint velocity.
-        upper_bound[2 * num_motors:3 * num_motors] = (motor.OBSERVED_TORQUE_LIMIT)  # Joint torque.
+        upper_bound[num_motors:2 * num_motors] = motor.MOTOR_SPEED_LIMIT  # Joint velocity.
+        upper_bound[2 * num_motors:3 * num_motors] = motor.OBSERVED_TORQUE_LIMIT  # Joint torque.
         upper_bound[3 * num_motors:] = 1.0  # Quaternion of base orientation.
         return upper_bound
 
