@@ -16,7 +16,7 @@ def getList():
 
 register(
     id='RexGalloping-v0',
-    entry_point='rex_gym.envs.gym.galloping_env:RexReactiveEnv',
+    entry_point='rex_gym.envs.gym.gallop_env:RexReactiveEnv',
     max_episode_steps=1000,
     reward_threshold=5.0,
 )
@@ -24,7 +24,7 @@ register(
 register(
     id='RexWalk-v0',
     entry_point='rex_gym.envs.gym.walk_env:RexWalkEnv',
-    max_episode_steps=1000,
+    max_episode_steps=2500,
     reward_threshold=5.0,
 )
 
@@ -41,3 +41,17 @@ register(
     max_episode_steps=400,
     reward_threshold=5.0,
 )
+
+register(
+    id='RexGo-v0',
+    entry_point='rex_gym.envs.gym.go_env:RexGoEnv',
+    max_episode_steps=1000,
+    reward_threshold=5.0,
+)
+
+register(
+    id='RexPoses-v0',
+    entry_point='rex_gym.envs.gym.poses_env:RexPosesEnv',
+    max_episode_steps=400,
+    reward_threshold=5.0,
+)
